/**
 *	@file	test.cpp
 *	@brief
 *	@author	tenk*
 *	@date	2010-??
 *	@note
 *		fname.h ̃eXg.
 */

#define USE_UNITST

#include "../disk.h"
#include "../disk_deep_mkdir.h"
#include "../TmpFile.h"
#include "unitst.h"

#ifdef _WIN32
#include <windows.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include <string>
#include "../disk_string.h"

#ifdef _WIN32
#include <atlbase.h>
#include <atlapp.h>
#include <atlmisc.h>
#include "../disk_cstring.h"
#endif

#define uint64_t	unsigned long long

#if 1
#include "../disk_fileload.h"
#else
void*	disk_fileLoad(const char* fname, void* mem, size_t size, size_t* pReadSize)
{
	assert(mem != NULL && size > 0);
	FILE *	fp = fopen(fname, _T("rb"));
	if (fp) {
		size_t rdSize = fread(mem, 1, size, fp);
		if (ferror(fp))
			mem = NULL;
		if (pReadSize)
			*pReadSize = rdSize;
		fclose(fp);
	} else {
		mem = NULL;
	}
	return mem;
}

bool disk_fileSave(const char* fname, const void* mem, size_t size)
{
	assert(mem != NULL);
	assert(size > 0);
	if (mem == 0 || size == 0)
		return false;
	FILE *	fp = fopen(fname, _T("wb"));
	if (fp == NULL)
		return false;
	size_t	n = fwrite(mem, 1, size, fp);
	if (ferror(fp))
		n = size - 1;
	fclose(fp);
	return n == size;
}
#endif

// ===========================================================================

UNITST_TEST(disk) {
	UNITST_CHECK( disk_fileExist("unitst.h") != 0 );
	UNITST_EQ( disk_fileSize("smp_64byte.txt"), 64 );
	unsigned char buf[0x10000];
	disk_fileLoad("smp_64byte.txt", buf, sizeof buf, NULL);
	unsigned bufSz = (unsigned)disk_fileSize("smp_64byte.txt");
	disk_fileSave("smp_64byte.bak", buf, bufSz);
	UNITST_EQ( disk_fileSize("smp_64byte.bak"), 64 );
	disk_time_t	ct = 0;
	disk_time_t	at = 0;
	disk_time_t	wt = 0;
	UNITST_EQ(disk_getTime( "smp_64byte.txt", &ct, &at, &wt), 0);
	UNITST_EQ(disk_setTime( "smp_64byte.bak",  ct,	at,  wt), 0);
	disk_time_t	ct2 = 0;
	disk_time_t	at2 = 0;
	disk_time_t	wt2 = 0;
	UNITST_EQ(disk_getTime( "smp_64byte.bak", &ct2, &at2, &wt2), 0);
  #ifdef _WIN32
	UNITST_EQ(ct, ct2);
  #endif
	UNITST_EQ(at, at2);
	UNITST_EQ(wt, wt2);

	disk_remove("tmp.bak");
	UNITST_CHECK( disk_rename("smp_64byte.bak", "tmp.bak") == 0 );

	char name[ DISK_MAX_PATH ];
	UNITST_CHECK( TmpFile_make(name, DISK_MAX_PATH, "test") == name );
	UNITST_CHECK( disk_fileExist( name) != 0 );

	disk_rmdir("test_dir");
	UNITST_EQ( disk_mkdir("test_dir") , 0 );
	UNITST_EQ( disk_chdir("test_dir") , 0 );
	disk_fileSave("smp.tmp", buf, bufSz);
	UNITST_EQ( disk_fileSize("smp.tmp") , 64 );
	UNITST_EQ( disk_chmod("smp.tmp", 0777) , 0 );
	UNITST_EQ( disk_remove("smp.tmp") , 0 );
	UNITST_NE( disk_getcwd(name, DISK_MAX_PATH), 0 );
	UNITST_EQ( disk_chdir("..") , 0 );
	UNITST_EQ( disk_rmdir("test_dir") , 0 );

	UNITST_EQ( disk_deep_mkdir("a/b/c/d/e") , 0);
	disk_fileSave("a/b/c/d/e/f.txt", buf, bufSz);
	UNITST_NE( disk_fileExist( "a/b/c/d/e/f.txt" ) , 0 );
  #ifdef DISK_FILELOAD_H
	std::vector<unsigned char> ttt;
	disk_fileLoad("smp_64byte.txt", ttt);
	disk_fileSave("a/b/c/d/tmp.tmp", ttt);
	UNITST_NE( disk_fileExist( "a/b/c/d/tmp.tmp" ) , 0 );
  #endif

	std::string fullpath_name = disk_fullpath("smp64byte.bak");

  #ifdef _WIN32
	std::string exe_name	  = disk_getExePath();
	//std::string exe_dir 	  = disk_getExeDir();
	std::string sys_dir 	  = disk_getSystemDir();
	std::string win_dir 	  = disk_getWindowsDir();

	CString fullpath_nameCS = disk_fullpathCS("smp64byte.bak");
	CString exe_nameCS		= disk_getExePathCS();
	//CString exe_dirCS 		= disk_getExeDirCS();
	CString sys_dirCS 		= disk_getSystemDirCS();
	CString win_dirCS 		= disk_getWindowsDirCS();
  #endif
}




// ===========================================================================

#if 1
/** C.
 */
int main()
{
	UNITST_RUN_ALL();
	return 0;
}
#endif
